"""Tests for podcust common tools."""

import pytest
from podcust.tools import get_user_input


# Mock input funtion:
# https://stackoverflow.com/a/56301794


class MockInputFunction:
    def __init__(self, return_value=None):
        self.return_value = return_value
        self._orig_input_fn = __builtins__["input"]

    def _mock_input_fn(self, prompt):
        print(prompt + ": " + str(self.return_value))
        return self.return_value

    def __enter__(self):
        __builtins__["input"] = self._mock_input_fn

    def __exit__(self, type, value, traceback):
        __builtins__["input"] = self._orig_input_fn


class TestGetUserInput:
    def test_validation_succeeds(self):
        with MockInputFunction(return_value="abc123"):
            assert get_user_input("test") == "abc123"

    def test_raises_length_error(self):
        with MockInputFunction(return_value="2" * 33):
            with pytest.raises(ValueError) as exc:
                get_user_input("test")

            assert str(exc.value) == ("Only type up to 30 characters.")

    def test_raises_non_alphanumeric(self):
        with MockInputFunction(return_value="a\r"):
            with pytest.raises(ValueError) as exc:
                get_user_input("test")

            assert str(exc.value) == ("Only type alphanumeric latin characters.")



"""Console script for podcust.

Useful documentation at:
https://click.palletsprojects.com/en/7.x/quickstart/#nesting-commands
https://click.palletsprojects.com/en/7.x/complex/
"""

import click
from podcust.demo.custodian import DemoCust
from podcust.transmission.custodian import TransmissionCust


@click.group()
def main(args=None):
    """Podcust commands provide a wrapper around lower level utilities
    from podman, the Operating System and the container you are managing."""
    click.echo("Welcome to Podman Custodian!")


@click.group()
@click.pass_context
def demo(ctx):
    """Podcust tools for demo container image."""
    # We can only use ctx.obj to create and share between commands.
    ctx.obj = DemoCust()
    click.echo("Initializing Podman Custodian Demo class.")


@click.command()
@click.pass_obj
def rmi(obj):
    """Remove a demo image."""
    click.echo("Removing Demo image.")
    obj.remove_stored_image()
    click.echo("Image removed!")


@click.command()
@click.pass_obj
def rmec(obj):
    """Remove an exited demo container."""
    click.echo("Removing Demo containers.")
    obj.removed_exited_containers()
    click.echo("Containers removed!")


@click.command()
@click.pass_obj
def build(obj):
    """Build a demo image."""
    click.echo("building Demo image.")
    obj.build_demo_image()
    click.echo("Image built!")


@click.command()
@click.pass_obj
def start(obj):
    """Start a demo container."""
    click.echo("Starting Demo container.")
    obj.run_container()
    click.echo("Demo container started!")


@click.command()
@click.pass_obj
def stop(obj):
    """Stop a demo container."""
    click.echo("Stopping Demo container.")
    obj.stop_container()
    click.echo("Demo container stopped!")


@click.command()
@click.pass_obj
def activate(obj):
    """Activate a demo container's service."""
    click.echo("Activating Demo container.")
    obj.activate_container()
    click.echo("Demo container service activated!")


@click.command()
@click.pass_obj
def deactivate(obj):
    """Deactivate a demo container's service."""
    click.echo("Deactivating Demo container.")
    obj.deactivate_container()
    click.echo("Demo container service deactivated!")


@click.command()
@click.pass_obj
def health(obj):
    """Health check of a demo container's service."""
    click.echo("Checking Demo container.")
    obj.health_check()


main.add_command(demo)
demo.add_command(rmi)
demo.add_command(rmec)
demo.add_command(build)
demo.add_command(start)
demo.add_command(stop)
demo.add_command(activate)
demo.add_command(deactivate)
demo.add_command(health)


@click.group()
@click.pass_context
def transmission(ctx):
    """Podcust tools for demo container image."""
    # We can only use ctx.obj to create and share between commands.
    ctx.obj = TransmissionCust()
    click.echo("Initializing Podman Custodian Transmission class.")


@click.command()
@click.pass_obj
def deploy(obj):
    """Deploy transmission container within a pod."""
    click.echo("Deploying transmission container within a pod")
    obj.deploy()
    click.echo("Transmission container deployed!")


main.add_command(transmission)
transmission.add_command(deploy)



"""
Retrieve Information about the platform the package is running on.
"""

import subprocess


def podman_exists():
    """
    Check that the podman package is installed and working properly.
    Raise an OSError if the podman --version command does not complete successfully.
    """

    check = subprocess.run(
        "podman --version",
        text=True,
        shell=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )

    if check.returncode != 0:
        raise OSError("podman package is not available!")



"""
Common podcust tools.
"""


def get_user_input(message: str):
    """
    Present a message to the user and capture user's response.
    Validates that the string is alphanumeric latin characters up to 30 length.
    (Wanting to avoid to have to deal with escapted characters and other weird input.
    Admittedly this should be trusted input but I m in favor of taking the precaution.
    30 alphanumeric characters should allow for complex enough passwords.)

    Args:
      message: Message to pring to the user.

    Returns:
      Reply from the user. Note that this is intended to be a trusted input.
    """

    output = input(message)

    if len(output) > 30:
        raise ValueError("Only type up to 30 characters.")
    elif not output.isalnum():
        raise ValueError("Only type alphanumeric latin characters.")
    else:
        return output



#!/usr/bin/env python3

"""Tests for `podcust` package."""

import pytest
import mock

from podcust.platform import podman_exists
from subprocess import CompletedProcess


class TestPodman:
    @mock.patch("podcust.platform.subprocess.run")
    def test_podman_exists1(self, mocked_run):
        mocked_run.return_value = CompletedProcess(
            args=["podman", "--version"],
            returncode=0,
            stdout="podman version 2.0.6\n",
            stderr="",
        )
        podman_exists()

    @mock.patch("podcust.platform.subprocess.run")
    def test_podman_exists2(self, mocked_run):
        mocked_run.return_value = CompletedProcess(
            args=["podman", "--version"],
            returncode=127,
            stdout="",
            stderr="--version: podman: command not found\n",
        )
        with pytest.raises(OSError, match="podman package is not available!"):
            podman_exists()



"""
Custodian Class for Demo container.
"""

from typing import List
import subprocess
from pathlib import Path, PurePath
import requests
from bs4 import BeautifulSoup  # type: ignore


class MultipleContainers(Exception):
    """
    Exception raised when more than one containers of a type are running.

    Inspired from:
    https://www.programiz.com/python-programming/user-defined-exception

    :param container_id1: First container id of container type
    :param container_id2: Second container id of container type.
        message -- explanation of the error
    """

    def __init__(self, container_id1, container_id2):
        self.container_id1 = container_id1
        self.container_id2 = container_id2
        self.message = (
            f"Duplicate container instance {container_id2} found while {container_id1}"
            " is also running."
        )
        super().__init__(self.message)


class MissingContainers(Exception):
    """
    Exception raised when not one containers of expected type are running.

    Inspired from:
    https://www.programiz.com/python-programming/user-defined-exception

    :param container_name: Name of container image.
        message -- explanation of the error
    """

    def __init__(self, container_name):
        self.message = f"No instance of {container_name} is running!"
        super().__init__(self.message)


class ContainerHealthError(Exception):
    """
    Exception raised when a container fails it's health check.

    Inspired from:
    https://www.programiz.com/python-programming/user-defined-exception

    :param container_name: Name of container image.
    :param container_id: Running Container id of container type container_name
        message -- explanation of the error
    """

    def __init__(self, container_name, container_id):
        self.message = (
            f"Failed health check for {container_name} with id: {container_id}"
        )
        super().__init__(self.message)


class DemoCust:
    """
    Main class for handling the httpdemo container.

    The httpdemo container just spawns an a container with an apache web server
    service serving the Fedora Test page through http.

    :param name: The Repository name of the image this class is custodian for.
    """

    name: str
    image_id: str

    def __init__(self, name: str = "localhost/httpdemo"):
        """
        Initialize DemoCust class.
        """
        self.name = name
        self.image_id = ""
        self.running_container_id = ""

    def find_stored_image_id(self) -> List[str]:
        """
        This function looks if the system has an appropriate container image and
        returns the id of that image.

        Current implementation assumes that the first match is the one we are after.

        TODO: Specify what tag we want to match?
        """

        image_id_list: List[str] = []
        check = subprocess.run(
            "podman images",
            text=True,
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )

        # split results line by line and remove all but one whitespace
        processed_lines = []
        for line in check.stdout.splitlines():
            # default split separator is spaces, too many spaces act as one separator
            tmp = " ".join(line.split())
            processed_lines.append(tmp.split(" "))

        # We expect the first line to have the columns below:
        # ['REPOSITORY', 'TAG', 'IMAGE', 'ID', 'CREATED', 'SIZE']
        for il in processed_lines:
            if il[0] == self.name:
                image_id_list.append(il[2])

        return image_id_list

    def remove_stored_image(self):
        """
        Removes a stored container image corresponding to the name
        the class has been instantiated to.
        """

        image_id_list = self.find_stored_image_id()
        command_text = "podman image rm $image_id"

        for iid in image_id_list:
            command_text = command_text.replace("$image_id", iid)
            print(f"Removing image {self.name} with image id {iid}")
            try:
                subprocess.run(
                    command_text,
                    text=True,
                    shell=True,
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE,
                    check=True,
                )
            except Exception as e:
                print(e)

    def find_exited_containers(self):  # noqa: E501
        """
        Find all containers of demo type that have run and exited.

        You can see here a sample output of the command we use to find the containers we want
        to remove:  # noqa: E501
        $ podman ps -a
        CONTAINER ID  IMAGE                                     COMMAND     CREATED     STATUS                   PORTS                 NAMES
        1b5fe6643ece  localhost/httpdemo:latest                 /sbin/init  2 days ago  Exited (0) 2 days ago    0.0.0.0:8080->80/tcp  strange_wu
        3ea6bf480c47  localhost/httpdemo:latest                 /bin/bash   2 days ago  Exited (0) 2 days ago    0.0.0.0:8080->80/tcp  funny_williams
        63604b048bc9  registry.fedoraproject.org/fedora:latest  /bin/bash   2 days ago  Exited (0) 2 days ago                          practical_kowalevski
        b3e4d5b363ce  localhost/httpdemo:latest                 /sbin/init  2 days ago  Exited (137) 2 days ago  0.0.0.0:8080->80/tcp  pedantic_tesla
        c4e4a6847c3d  localhost/httpdemo:latest                 /bin/bash   2 days ago  Exited (0) 2 days ago    0.0.0.0:8080->80/tcp  jolly_volhard
        dc9bffeef1c2  registry.fedoraproject.org/fedora:latest  /bin/bash   2 days ago  Exited (0) 2 days ago    0.0.0.0:8080->80/tcp  zealous_blackburn0
        """

        container_id_list: List[str] = []
        check = subprocess.run(
            "podman ps -a",
            text=True,
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            check=True,
        )

        # split results line by line and remove all but one whitespace
        processed_lines = []
        for line in check.stdout.splitlines():
            # default split separator is spaces, too many spaces act as one separator
            tmp = " ".join(line.split())
            processed_lines.append(tmp.split(" "))

        # We expect the first line to have the columns below:
        # ['CONTAINER ID', 'IMAGE', 'COMMAND', 'CREATED', 'CREATED', 'STATUS', 'PORTS', 'NAMES']
        for il in processed_lines:
            container_name = il[1].split(":")[0]
            status = il[6]

            if container_name == self.name and status == "Exited":
                container_id_list.append(il[0])

        return container_id_list

    def removed_exited_containers(self):
        """
        Remove all containers of demo type that have run and exited.

        Remove an image with:
        podman container rm 3ea6bf480c47
        """

        container_id_list = self.find_exited_containers()

        for cont in container_id_list:

            try:
                print(f"Removing container {self.name} with container id {cont}")
                command_text = "podman container rm $container_id"
                command_text = command_text.replace("$container_id", cont)
                subprocess.run(
                    command_text,
                    text=True,
                    shell=True,
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE,
                    check=True,
                )

            except Exception as e:
                print(e)

    def build_demo_image(self):
        """
        Build an image for the demo container. Use the dockerfile located at this folder.

        The command to build a container is:
        podman build -f Dockerfile -t httpdemo
        """

        command_text = "podman build -f $dockerfile -t httpdemo"
        dockerfile_dir = str(PurePath.joinpath(Path(__file__).parent, "Dockerfile"))
        command_text = command_text.replace("$dockerfile", dockerfile_dir)

        try:
            p = subprocess.run(
                command_text,
                text=True,
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                check=True,
            )
            print(p.stdout)

        except Exception as e:
            print(e)

    def run_container(self):
        """
        Start running the demo container.
        """
        command_text = "podman run -d -p 8080:80 localhost/httpdemo"
        try:
            p = subprocess.run(
                command_text,
                text=True,
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                check=True,
            )
            print(p.stdout)
            self.running_container_id = p.stdout

        except Exception as e:
            print(e)

    def get_running_container_id(self):
        """
        Get the container ID for a running container (of demo type).
        """
        command_text = "podman ps"
        p = subprocess.run(
            command_text,
            text=True,
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            check=True,
        )

        # split results line by line and remove all but one whitespace
        processed_lines = []
        for line in p.stdout.splitlines():
            # default split separator is spaces, too many spaces 'act' as one separator
            tmp = " ".join(line.split())
            processed_lines.append(tmp.split(" "))

        # We expect the first line to have the columns below:
        # ['CONTAINER ID', 'IMAGE', 'COMMAND', 'CREATED', 'CREATED', 'STATUS', 'PORTS', 'NAMES']
        for il in processed_lines:
            container_name = il[1].split(":")[0]
            status = il[6]

            if container_name == self.name and status == "Up":
                if self.running_container_id == "":
                    self.running_container_id = il[0]
                else:
                    raise MultipleContainers(self.running_container_id, il[0])
        # raise error if no container is found
        if self.running_container_id == "":
            raise MissingContainers(self.name)

    def stop_container(self):
        """
        Stop demo running container.
        """
        command_text: str = "podman kill $container_id"
        # if container_id not register, retrieve it
        if self.running_container_id == "":
            self.get_running_container_id()

        command_text = command_text.replace("$container_id", self.running_container_id)

        try:
            subprocess.run(
                command_text,
                text=True,
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                check=True,
            )
            self.running_container_id = ""

        except Exception as e:
            print(e)

    def activate_container(self):
        """
        Activate a demo's container httpd service.
        """
        command_text: str = "podman exec $container_id systemctl start httpd"
        # if container_id not register, retrieve it
        if self.running_container_id == "":
            self.get_running_container_id()

        command_text = command_text.replace("$container_id", self.running_container_id)

        try:
            subprocess.run(
                command_text,
                text=True,
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                check=True,
            )
            self.running_container_id = ""

        except Exception as e:
            print(e)

    def deactivate_container(self):
        """
        Deactivate a demo's container httpd service.
        """
        command_text: str = "podman exec $container_id systemctl stop httpd"
        # if container_id not register, retrieve it
        if self.running_container_id == "":
            self.get_running_container_id()

        command_text = command_text.replace("$container_id", self.running_container_id)

        try:
            subprocess.run(
                command_text,
                text=True,
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                check=True,
            )
            self.running_container_id = ""

        except Exception as e:
            print(e)

    def health_check(self):
        """
        Runs basic checks to test container's functionality.

        Verifying health check inspired by:
        https://stackoverflow.com/a/51242/1904901
        """

        req = requests.get("http://127.0.0.1:8080/")

        # parse the content
        soup = BeautifulSoup(req.text, features="html5lib")
        title = soup.find("title")

        if title.text == "Test Page for the HTTP Server on Fedora":
            print("Health check succeeded!")
        else:
            raise ContainerHealthError(self.name, self.running_container_id)



"""
Unit Tests for DemoCust class.
"""
from podcust.demo import custodian
import mock
from subprocess import CompletedProcess
import pytest


class TestDemoCust:
    def setup_class(cls):
        cls.demo = custodian.DemoCust()

    @mock.patch("podcust.demo.custodian.subprocess.run")
    def test_find_stored_image(self, mocked_run):
        """
        Example output from podman images:
        REPOSITORY                         TAG     IMAGE ID      CREATED       SIZE
        localhost/httpdemo                 latest  5ff443b54997  46 hours ago  582 MB

        The hardcoded result needs to be updated properly, from a an eralier run test!
        """
        mocked_run.return_value = CompletedProcess(
            args="podman images",
            returncode=0,
            stdout=(
                "REPOSITORY                         TAG     IMAGE ID      CREATED       SIZE\n"
                "localhost/httpdemo                 latest  5ff443b54997  46 hours ago  582 MB\n"
                "registry.fedoraproject.org/fedora  latest  00ff39a8bf19  2 months ago  189 MB"
            ),
            stderr="",
        )
        result = self.demo.find_stored_image_id()
        assert result == ["5ff443b54997"]

    @mock.patch("podcust.demo.custodian.DemoCust.find_stored_image_id")
    @mock.patch("podcust.demo.custodian.subprocess.run")
    def test_remove_stored_image(self, mocked_run, mocked_image):
        """"""
        mocked_image.return_value = ["5ff443b54997"]
        self.demo.remove_stored_image()
        mocked_run.assert_called_with(
            "podman image rm 5ff443b54997",
            text=True,
            shell=True,
            stdout=-1,
            stderr=-1,
            check=True,
        )

    @mock.patch("podcust.demo.custodian.subprocess.run")
    def test_find_exited_containers(self, mocked_run):
        """
        Example output for container images:
        $ podman ps -a  # noqa: E501
        CONTAINER ID  IMAGE                                     COMMAND     CREATED     STATUS                   PORTS                 NAMES
        1b5fe6643ece  localhost/httpdemo:latest                 /sbin/init  2 days ago  Exited (0) 2 days ago    0.0.0.0:8080->80/tcp  strange_wu
        3ea6bf480c47  localhost/httpdemo:latest                 /bin/bash   2 days ago  Exited (0) 2 days ago    0.0.0.0:8080->80/tcp  funny_williams
        63604b048bc9  registry.fedoraproject.org/fedora:latest  /bin/bash   2 days ago  Exited (0) 2 days ago                          practical_kowalevski
        b3e4d5b363ce  localhost/httpdemo:latest                 /sbin/init  2 days ago  Exited (137) 2 days ago  0.0.0.0:8080->80/tcp  pedantic_tesla
        c4e4a6847c3d  localhost/httpdemo:latest                 /bin/bash   2 days ago  Exited (0) 2 days ago    0.0.0.0:8080->80/tcp  jolly_volhard
        dc9bffeef1c2  registry.fedoraproject.org/fedora:latest  /bin/bash   2 days ago  Exited (0) 2 days ago    0.0.0.0:8080->80/tcp  zealous_blackburn0


        The hardcoded result needs to be updated properly, from a an eralier run test!
        """
        mocked_run.return_value = CompletedProcess(
            args="podman images",
            returncode=0,
            stdout=(  # noqa: E501
                "CONTAINER ID  IMAGE                                     COMMAND     CREATED     STATUS                   PORTS                 NAMES\n"  # noqa: E501
                "1b5fe6643ece  localhost/httpdemo:latest                 /sbin/init  2 days ago  Exited (0) 2 days ago    0.0.0.0:8080->80/tcp  strange_wu\n"  # noqa: E501
                "63604b048bc9  registry.fedoraproject.org/fedora:latest  /bin/bash   2 days ago  Exited (0) 2 days ago                          practical_kowalevski"  # noqa: E501
            ),
            stderr="",
        )
        result = self.demo.find_exited_containers()
        assert result == ["1b5fe6643ece"]

    @mock.patch("podcust.demo.custodian.DemoCust.find_exited_containers")
    @mock.patch("podcust.demo.custodian.subprocess.run")
    def test_remove_exited_containers(self, mocked_run, mocked_ec):
        """"""
        mocked_ec.return_value = ["5ff443b54997"]
        self.demo.removed_exited_containers()
        mocked_run.assert_called_with(
            "podman container rm 5ff443b54997",
            text=True,
            shell=True,
            stdout=-1,
            stderr=-1,
            check=True,
        )

    @mock.patch("podcust.demo.custodian.__file__", "/home/user/file.py")
    @mock.patch("podcust.demo.custodian.subprocess.run")
    def test_demo_build_demo_image(self, mocked_run):
        """"""
        self.demo.build_demo_image()
        mocked_run.assert_called_with(
            "podman build -f /home/user/Dockerfile -t httpdemo",
            text=True,
            shell=True,
            stdout=-1,
            stderr=-1,
            check=True,
        )

    @mock.patch("podcust.demo.custodian.subprocess.run")
    def test_get_running_container_id(self, mocked_run):
        """"""

        mocked_run.return_value = CompletedProcess(
            args="podman ps",
            returncode=0,
            stdout=(
                "CONTAINER ID  IMAGE                      COMMAND     CREATED         STATUS             PORTS                 NAMES\n"  # noqa: E501
                "71440253d707  localhost/httpdemo:latest  /sbin/init  21 minutes ago  Up 21 minutes ago  0.0.0.0:8080->80/tcp  pedantic_chatelet\n"  # noqa: E501
            ),
            stderr="",
        )

        self.demo.get_running_container_id()
        assert self.demo.running_container_id == "71440253d707"

    @mock.patch("podcust.demo.custodian.subprocess.run")
    def test_get_running_container_id_exception(self, mocked_run):
        """

        Note:
        The test works because the class has been populated with the running id from
        the previous test test_get_running_container_id.
        """

        mocked_run.return_value = CompletedProcess(
            args="podman ps",
            returncode=0,
            stdout=(
                "CONTAINER ID  IMAGE                      COMMAND     CREATED         STATUS             PORTS                 NAMES\n"  # noqa: E501
                "71440253d708  localhost/httpdemo:latest  /sbin/init  21 minutes ago  Up 21 minutes ago  0.0.0.0:8080->80/tcp  pedantic_chatelet\n"  # noqa: E501
                "71440253d709  localhost/httpdemo:latest  /sbin/init  21 minutes ago  Up 21 minutes ago  0.0.0.0:8080->80/tcp  pedantic_chatelet\n"  # noqa: E501
            ),
            stderr="",
        )

        with pytest.raises(custodian.MultipleContainers) as exc:
            self.demo.get_running_container_id()

        assert str(exc.value) == (
            "Duplicate container instance 71440253d708 found while 71440253d707 is also running."
        )

    @mock.patch("podcust.demo.custodian.subprocess.run")
    def test_run_container(self, mocked_run):
        """
        Note: Because this test assigns running_container_id it has to run after
        test_demo_get_running_container_id and test_demo_get_running_container_id_exception.
        """

        mocked_run.return_value = CompletedProcess(
            args="podman run -d -p 8080:80 localhost/httpdemo",
            returncode=0,
            stdout=("71440253d707c083ddfaf1e47b3bf9db37d3d0189d232237daac128f4a4aa9f0"),
            stderr="",
        )

        self.demo.run_container()
        assert (
            self.demo.running_container_id
            == "71440253d707c083ddfaf1e47b3bf9db37d3d0189d232237daac128f4a4aa9f0"
        )

    @mock.patch("podcust.demo.custodian.subprocess.run")
    def test_stop_container(self, mocked_run):
        """"""
        self.demo.stop_container()
        mocked_run.assert_called_with(
            "podman kill 71440253d707c083ddfaf1e47b3bf9db37d3d0189d232237daac128f4a4aa9f0",
            text=True,
            shell=True,
            stdout=-1,
            stderr=-1,
            check=True,
        )

    @mock.patch("podcust.demo.custodian.subprocess.run")
    def test_activate_container(self, mocked_run):
        """"""
        self.demo.running_container_id == "71440253d708"
        mocked_run.return_value = CompletedProcess(
            args="podman ps",
            returncode=0,
            stdout=(
                "CONTAINER ID  IMAGE                      COMMAND     CREATED         STATUS             PORTS                 NAMES\n"  # noqa: E501
                "71440253d708  localhost/httpdemo:latest  /sbin/init  21 minutes ago  Up 21 minutes ago  0.0.0.0:8080->80/tcp  pedantic_chatelet\n"  # noqa: E501
            ),
            stderr="",
        )
        self.demo.activate_container()
        mocked_run.assert_called_with(
            "podman exec 71440253d708 systemctl start httpd",
            text=True,
            shell=True,
            stdout=-1,
            stderr=-1,
            check=True,
        )

    @mock.patch("podcust.demo.custodian.subprocess.run")
    def test_deactivate_container(self, mocked_run):
        """"""
        self.demo.running_container_id == "71440253d708"
        mocked_run.return_value = CompletedProcess(
            args="podman ps",
            returncode=0,
            stdout=(
                "CONTAINER ID  IMAGE                      COMMAND     CREATED         STATUS             PORTS                 NAMES\n"  # noqa: E501
                "71440253d708  localhost/httpdemo:latest  /sbin/init  21 minutes ago  Up 21 minutes ago  0.0.0.0:8080->80/tcp  pedantic_chatelet\n"  # noqa: E501
            ),
            stderr="",
        )
        self.demo.deactivate_container()
        mocked_run.assert_called_with(
            "podman exec 71440253d708 systemctl stop httpd",
            text=True,
            shell=True,
            stdout=-1,
            stderr=-1,
            check=True,
        )


class TestDemoCust2:
    def setup_class(cls):
        cls.demo = custodian.DemoCust()

    @mock.patch("podcust.demo.custodian.subprocess.run")
    def test_get_running_container_id_missing(self, mocked_run):
        """"""

        mocked_run.return_value = CompletedProcess(
            args="podman ps",
            returncode=0,
            stdout=(
                "CONTAINER ID  IMAGE                      COMMAND     CREATED         STATUS             PORTS                 NAMES\n"  # noqa: E501
                "71440253d707  localhost/other:latest  /sbin/init  21 minutes ago  Up 21 minutes ago  0.0.0.0:8080->80/tcp  pedantic_chatelet\n"  # noqa: E501
                "71440253d708  localhost/other:latest  /sbin/init  21 minutes ago  Up 21 minutes ago  0.0.0.0:8080->80/tcp  pedantic_chatelet\n"  # noqa: E501
            ),
            stderr="",
        )

        with pytest.raises(custodian.MissingContainers) as exc:
            self.demo.get_running_container_id()

        assert str(exc.value) == "No instance of localhost/httpdemo is running!"



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages
import os
# Needs pip>=20
from pip._internal.network.session import PipSession
from pip._internal.req import parse_requirements

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()
# adapted from:
# https://stackoverflow.com/a/57191701
# https://stackoverflow.com/a/59969843
# https://alexanderwaldin.github.io/packaging-python-project.html
requirements0 = parse_requirements(
    os.path.join(os.path.dirname(__file__), 'requirements.txt'),
    session=PipSession()
)
requirements = [str(requirement.requirement) for requirement in requirements0]
del requirements0


setup_requirements = [
    "pip>=20.2",
    "bump2version>=1.0",
    "wheel>=0.35.1",
    "Sphinx>=3.2.1",
    "twine>=3.2.0",
    "check-manifest"
]

test_requirements = [
    'pytest',
    'pytest-runner',
    "mock",
    "flake8",
    "black",
    "mypy",
    "tox",
    "coverage",
    "yamllint",
    "doc8"
]

setup(
    author="Nikolaos Perrakis",
    author_email='nikperrakis@gmail.com',
    python_requires='>=3.8',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
    ],
    description="Python utility to handle podman containers within Fedora.",
    entry_points={
        'console_scripts': [
            'podcust=podcust.cli:main',
        ],
    },
    install_requires=requirements,
    license="GNU General Public License v3",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='podcust',
    name='podcust',
    packages=find_packages(include=['podcust', 'podcust.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    # hack from https://stackoverflow.com/a/41398850/1904901 to be able to install deps from pip
    extras_require={
        "dev": setup_requirements + test_requirements
    },
    # Adding man pages to python package: https://stackoverflow.com/a/48381612/1904901
    data_files = [('/usr/share/man/man1/', ['docs/_build/man/podcust.1'])],
    url='https://github.com/Iolaum/podcust',
    version='0.0.29',
    zip_safe=False,
)



"""
Custodian Class for Transmission container.

Using the `Linuxserver.io transmission`_ `container image`_

.. _`container image`: https://github.com/linuxserver/docker-transmission
.. _`Linuxserver.io transmission`: https://docs.linuxserver.io/images/docker-transmission

This module is responsible for setting up and maintaining the container.

The container is intended to live inside the users home directory, in a hardcoded
location. Namely ``$HOME/transmission/``. The container is deployed within a pod
named transmission.
"""

# from typing import List
from shutil import rmtree
import subprocess
import getpass
from pathlib import Path
import importlib.resources as pkg_resources
from podcust.tools import get_user_input


class TransmissionCust:
    """
    Main class for handling the transmission container.

    :param name: The full repository name of the image this class is custodian for.
    """

    name: str
    image_id: str
    username: str

    def __init__(self, name: str = "ghcr.io/linuxserver/transmission"):
        """
        Initialize TransmissionCust class.
        """
        self.name = name
        self.image_id = ""
        self.username = getpass.getuser()

    def pull_latest_transmission_image(self):
        """
        Pull latest transmission container image from linuxserver.io

        The relevant shell command is::

        $ podman pull ghcr.io/linuxserver/transmission

        """

        command_text = "podman pull $name"
        command_text = command_text.replace("$name", self.name)

        try:
            p = subprocess.run(
                command_text,
                text=True,
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                check=True,
            )
            print(p.stdout)

        except Exception as e:
            print(e)

    def deploy(self):
        """
        Create a pod named transmission to deploy our container.
        For a successful deployment the following steps must take place:

        - Create the necessary folders, and give them proper permissions.
        - Write the proper kube yaml file that we 'll use to deploy the container.
        - Open the necessary firewall port.
        - Execute the podman play command to start the pod with the transmission container.
        """

        # Let's create master path where we 'll put the container
        main_path: Path = Path.home().joinpath("transmission")

        # Let's delete everything allready present on the space we want to use:
        if main_path.exists():
            rmtree(main_path)

        # let's recreate the directory now
        print("Creating key directories:")
        main_path.mkdir()
        # let's create additional needed directories
        config_dir = main_path.joinpath("config")
        config_dir.mkdir()
        watch_dir = main_path.joinpath("watch")
        watch_dir.mkdir()
        downloads_dir = main_path.joinpath("downloads")
        downloads_dir.mkdir()

        # We also need to change the SELinux security context of the new directories:
        print("Setting proper directory permissions:")
        command_text = "chcon -u system_u " + str(config_dir)
        subprocess.run(
            command_text,
            text=True,
            shell=True,
            check=True,
        )
        command_text = "chcon -u system_u " + str(watch_dir)
        subprocess.run(
            command_text,
            text=True,
            shell=True,
            check=True,
        )
        command_text = "chcon -u system_u " + str(downloads_dir)
        subprocess.run(
            command_text,
            text=True,
            shell=True,
            check=True,
        )
        command_text = "chcon -r object_r " + str(config_dir)
        subprocess.run(
            command_text,
            text=True,
            shell=True,
            check=True,
        )
        command_text = "chcon -r object_r " + str(watch_dir)
        subprocess.run(
            command_text,
            text=True,
            shell=True,
            check=True,
        )
        command_text = "chcon -r object_r " + str(downloads_dir)
        subprocess.run(
            command_text,
            text=True,
            shell=True,
            check=True,
        )
        command_text = "chcon -t container_file_t " + str(config_dir)
        subprocess.run(
            command_text,
            text=True,
            shell=True,
            check=True,
        )
        command_text = "chcon -t container_file_t " + str(watch_dir)
        subprocess.run(
            command_text,
            text=True,
            shell=True,
            check=True,
        )
        command_text = "chcon -t container_file_t " + str(downloads_dir)
        subprocess.run(
            command_text,
            text=True,
            shell=True,
            check=True,
        )
        command_text = "podman unshare chown -R 1000:1000 " + str(config_dir)
        subprocess.run(
            command_text,
            text=True,
            shell=True,
            check=True,
        )
        command_text = "podman unshare chown -R 1000:1000 " + str(watch_dir)
        subprocess.run(
            command_text,
            text=True,
            shell=True,
            check=True,
        )
        command_text = "podman unshare chown -R 1000:1000 " + str(downloads_dir)
        subprocess.run(
            command_text,
            text=True,
            shell=True,
            check=True,
        )

        # read package file
        # https://stackoverflow.com/a/20885799/1904901
        # To access a file inside the current module, set the package argument to __package__,
        yaml_template = pkg_resources.read_text(
            __package__, "transmission-kube-template.yml"
        )

        # populate template with proper entries:
        usnmae = get_user_input("Enter transmission username:")
        yaml_template = yaml_template.replace("$SET_USER", usnmae)
        password = get_user_input("Enter transmission password:")
        yaml_template = yaml_template.replace("$SET_PASSWD", password)
        yaml_template = yaml_template.replace("$LOCAL_USER", self.username)

        # write kubernetes template
        kube_yaml_path = main_path.joinpath("transmission-kube.yml")
        kube_yaml_path.write_text(yaml_template)

        subprocess.run(
            "podman play kube " + str(kube_yaml_path),
            text=True,
            shell=True,
            check=True,
        )

        # open necessary firewall port
        # sudo firewall-cmd --permanent --add-port=9091/tcp
        # There is a bug in Fedora IoT that requires this command to be run under sudo
        # Fedora docs for handling firewalld commands:
        # https://docs.fedoraproject.org/en-US/quick-docs/firewalld/
        subprocess.run(
            "sudo firewall-cmd --permanent --add-port=9091/tcp",
            text=True,
            shell=True,
            check=True,
        )
